//
//  LinkvPlayerController.h
//  LinkvPlayer
//
//  Created by SuZhou on 2020/6/2.
//  Copyright © 2020年 linkv. All rights reserved.
//

#import <AVFoundation/AVFoundation.h>
#import <MediaPlayer/MediaPlayer.h>
#import <UIKit/UIKit.h>
#import "LinkvPlayerDef.h"
#import "PlayQosInfo.h"
#import "LinkvDelegate.h"
#import "LinkvConfig.h"



@interface LinkvPlayerController : NSObject

/// 初始化SDK
/// @param key 请联系技术支持获取，测试阶段可传空
/// @param env  false：正式环境 true：测试环境
+ (void)initSDK:(NSString *) key debugEnv:(BOOL)env;

/// 获取播放默认配置参数
+ (LinkvConfig *)getDefaultConfig;
/**
 @abstract 初始化播放器并设置播放地址
 @param config 播放相关配置
 @return 返回LinkvPlayerController 实例
 */
- (instancetype)initWithConfig:(LinkvConfig *)config;

/**
 @abstract 正在播放的视频文件的URL地址，该地址可以是本地地址或者服务器地址。
 */
@property (nonatomic, readonly) NSString *contentURL;

/**
 @abstract 包含视频播放内容的VIEW（只读）。
 @discussion view的使用逻辑：
 
 * 可以通过frame设置view大大小
 * 使用[scalingMode]([LinkvPlayerController scalingMode]) 可以更改视频内容在VIEW中的显示情况
 
 @see scalingMode
 */
// The view in which the media and playback controls are displayed.
@property (nonatomic, readonly) UIView *view;


@property (nonatomic, weak) id<LinkvDelegate> delegate;

/**
 @abstract 当前网络加载情况
 @discussion 可以通过该属性获取视频加载情况：
 
 <pre><code>
 typedef NS_OPTIONS(NSUInteger, MPMovieLoadState) {
 MPMovieLoadStateUnknown        = 0,        // 加载情况未知
 MPMovieLoadStatePlayable       = 1 << 0,   // 加载完成，可以播放
 MPMovieLoadStatePlaythroughOK  = 1 << 1,   // 加载完成
 MPMovieLoadStateStalled        = 1 << 2,   // 如果视频正在加载中
 } NS_DEPRECATED_IOS(3_2, 9_0);
 </code></pre>
 @discussion 通知：
 
 * MPMoviePlayerLoadStateDidChangeNotification，当加载状态变化时提供通知
*/
// Returns the network load state of the movie player.
@property (nonatomic, readonly) MPMovieLoadState loadState;



/**
 @abstract 当前缩放显示模式。
 @discussion 当前支持四种缩放模式：
 
 <pre><code>
 typedef NS_ENUM(NSInteger, MPMovieScalingMode) {
 MPMovieScalingModeNone,       // 无缩放
 MPMovieScalingModeAspectFit,  // 同比适配，某个方向会有黑边
 MPMovieScalingModeAspectFill, // 同比填充，某个方向的显示内容可能被裁剪
 MPMovieScalingModeFill        // 满屏填充，与原始视频比例不一致
 } NS_DEPRECATED_IOS(2_0, 9_0);
 </code></pre>
 
 */
// Determines how the content scales to fit the view. Defaults to MPMovieScalingModeAspectFit.
@property (nonatomic) MPMovieScalingMode scalingMode;

/**
 @abstract 当前视频总时长
 @discussion 视频总时长，单位是秒。
 
 * 如果是直播视频源，总时长为0.
 * 如果该信息未知，总时长默认为0.
 
 */
// The duration of the movie, or 0.0 if not known.
@property (nonatomic, readonly) NSTimeInterval duration;

/**
 @abstract 当前视频可播放长度
 @discussion 视频可播放时长，单位是秒。
 
 * currentPlaybackTime 标记的是播放器当前已播放的时长。
 * playableDuration 标记的是播放器缓冲的时间，会稍大于currentPlaybackTime，与currentPlaybackTime的差值则是缓冲长度。
 * duration 是视频总时长。
 
 */
// The currently playable duration of the movie, for progressively downloaded network content.
@property (nonatomic, readonly) NSTimeInterval playableDuration;


/**
 @abstract 当前视频宽高
 @discussion 获取信息
 
 * 监听MPMovieNaturalSizeAvailableNotification
 * 播放过程中，宽高信息可能会产生更改
 
 */
@property (nonatomic, readonly) CGSize naturalSize;


/**
 @abstract bufferTimeMax指定播放时的缓冲时长，单位秒
 @discussion 对于直播流，该属性用于直播延时控制；对于点播流，该属性用于缓冲时长控制
 
 * 直播流该属性默认为2秒，设置为0或负值时为关闭直播追赶
 * 点播流该属性默认为3600秒，且与bufferSizeMax同时生效，两者取小值
 */
@property NSTimeInterval bufferTimeMax;


/**
 @abstract buffer为空时，拉取数据所耗的时长
 @discussion 当buffer为空时，开始统计。单位为秒。
 
 * 当MPMoviePlayerLoadStateDidChangeNotification 通知发起；
 * MPMovieLoadState状态为MPMovieLoadStateStalled 开始计时；
 * MPMovieLoadState状态为MPMovieLoadStatePlayable 或者 MPMovieLoadStatePlaythroughOK时，结束计时；
 */
@property (nonatomic, readonly) NSTimeInterval bufferEmptyDuration;

/**
 @abstract 发起cache的次数
 @discussion 当buffer为空时，统计一次，统计的条件为
 
 * 当MPMoviePlayerLoadStateDidChangeNotification 通知发起
 * MPMovieLoadState 状态为MPMovieLoadStateStalled
 */
@property (nonatomic, readonly) NSInteger bufferEmptyCount;

/**
 @abstract 视频流server ip
 @discussion 当收到prepared后，即可以查询当前连接的视频流server ip
 */
@property (nonatomic, readonly) NSString* serverAddress;

/**
 @abstract 视频流qos信息
 @discussion 在播放过程中，即可以查询当前连接的视频流qos信息
 */
@property (nonatomic, strong) PlayQosInfo *qosInfo;

/**
 @abstract 是否静音
 @discussion
 * 默认不静音
 */
@property(nonatomic) BOOL shouldMute;

/**
 @abstract 获取sdk版本
 */
+ (NSString *)getVersion;

/**
 @abstract 当前播放器是否在播放
 @return 正在播放返回TRUE。其他情况返回FASLE。
 @see playbackState
 */
- (BOOL)isPlaying;


/**
 @abstract 获取当前播放的pts
 */
- (NSTimeInterval)getCurrentPts;


/// 设置播放地址，开始播放
/// @param url 播放地址
- (void)setDataSource:(NSString *)url;


/// 结束播放
- (void)stop;

/**
 @abstract 播放视频的当前时刻，单位为秒。
 @discussion currentPlaybackTime属性更改时机：
 */
// The current playback time of the now playing item in seconds.
@property(nonatomic) NSTimeInterval currentPlaybackTime;
@end
